/**
 * @file boost/mirror/meta_meta_object.hpp
 * @brief Reflection of meta-objects
 *
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_MIRROR_META_META_OBJECT_1011291729_HPP
#define BOOST_MIRROR_META_META_OBJECT_1011291729_HPP

#include <boost/mirror/mirror_fwd.hpp>
#include <boost/mirror/meta_object_tags.hpp>
#include <boost/mirror/preprocessor.hpp>
#include <boost/mirror/auxiliary/scoped_named.hpp>
#include <boost/mirror/meta_prog/range.hpp>
#include <boost/mirror/ct_string/basic_string.hpp>

BOOST_MIRROR_NAMESPACE_BEGIN
namespace aux {

template <typename Tag>
struct meta_meta_object_base;

#define BOOST_MIRROR_HLPR_MAKE_META_OBJECT_NICE_NAME(OBJECT, PREFIX) \
        PREFIX #OBJECT

#define BOOST_MIRROR_HLPR_META_OBJECT_TAG(OBJECT, X) \
        meta_ ## OBJECT ## _tag

#define BOOST_MIRROR_HLPR_IMPLEMENT_MMO_NAMES(OBJECT, I, X) \
template <> \
struct meta_meta_object_base<meta_## OBJECT ##_tag> \
{ \
protected: \
        struct ctrt \
        { \
                BOOST_MIRROR_IMPLEMENT_META_OBJECT_NAME_FUNCTIONS(#OBJECT) \
        }; \
        struct base \
        { \
                BOOST_MIRROR_IMPLEMENT_META_OBJECT_NAME_FUNCTIONS( \
                        BOOST_MIRROR_META_OBJECT_NICE_NAME( \
                                OBJECT, \
                                BOOST_MIRROR_HLPR_MAKE_META_OBJECT_NICE_NAME, \
                                "Meta" \
                        ) \
                ) \
        };\
public: \
        typedef boost::mirror::mp::range< \
                BOOST_MIRROR_FOR_EACH_META_OBJECT_BASE( \
                        OBJECT, \
                        BOOST_MIRROR_HLPR_META_OBJECT_TAG, \
                        _, \
                        BOOST_MIRROR_PP_COMMA \
                ) \
        > gens; \
};

BOOST_MIRROR_FOR_EACH_META_OBJECT(BOOST_MIRROR_HLPR_IMPLEMENT_MMO_NAMES, _)
#undef BOOST_MIRROR_HLPR_MAKE_META_OBJECT_NICE_NAME
#undef BOOST_MIRROR_HLPR_IMPLEMENT_MMO_NAMES

template <typename Range>
struct wrap_mmo_gens;

template <typename ... T>
struct wrap_mmo_gens<boost::mirror::mp::range<T...> >
{
        typedef boost::mirror::mp::range<
                boost::mirror::meta_meta_object<T>...
        > type;
};

} // namespace aux

template <typename MetaObjectCategory>
struct meta_meta_object : aux::meta_meta_object_base<
        MetaObjectCategory
>
{
public:
        // The category of the reflected meta-object
        typedef MetaObjectCategory category;
private:
        typedef aux::meta_meta_object_base< category > base_class;

        static std::string make_ctrt_name(std::string res)
        {
                std::string::iterator i = res.begin(), e = res.end();
                while(i != e)
                {
                        if(*i == '_') *i = ' ';
                        ++i;
                }
                return res;
        }
public:
        typedef typename boost::mirror::cts::string<>::type static_name;

        // Returns the base name of the meta-object
        static std::string base_name(void)
        {
                return std::string(
                        base_class::base::name(),
                        base_class::base::name_length()
                );
        }

        // Returns the name of the construct reflected by the meta-object
        static std::string construct_name(void)
        {
                static std::string res = make_ctrt_name(
                        std::string(
                                base_class::ctrt::name(),
                                base_class::ctrt::name_length()
                        )
                );
                return res;
        }
};

template <typename MetaObjectCategory>
struct generalizations<meta_meta_object<MetaObjectCategory> >
{
        typedef typename boost::mirror::aux::wrap_mmo_gens<
                typename boost::mirror::aux::meta_meta_object_base<
                        MetaObjectCategory
                >::gens
        >::type type;
};

#ifdef BOOST_MIRROR_DOCUMENTATION_ONLY
/// Reflects the meta_object passed as the @a META_OBJECT argument
/**
 *  This macro expands into a type conforming to the MetaMetaObject
 *  concept, which provides meta-data about the given Mirror's MetaObject.
 *  The type name passed as the @a META_TYPE argument
 *  must by a valid Mirror's meta-object.
 *
 *  @see boost::mirror::MetaObject
 *  @see boost::mirror::MetaMetaObject
 *  @see boost::lagoon::meta_meta_object
 *  @see boost::lagoon::meta_object::self
 *
 *  @ingroup reflection_macros
 */
#define BOOST_MIRRORED_META_OBJECT(META_OBJECT) boost::mirror::MetaMetaObject
#else
#define BOOST_MIRRORED_META_OBJECT(META_OBJECT) \
        boost::mirror::meta_meta_object< \
                typename boost::mirror::meta_object_category< META_OBJECT >::type \
        >
#endif


BOOST_MIRROR_NAMESPACE_END

#endif //include guard

