/**
 * .file boost/puddle/auxiliary/ct_string.hpp
 * .brief The Mirror compile-time string wrapper
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_PUDDLE_AUXILIARY_CT_STRING_1106061232_HPP
#define BOOST_PUDDLE_AUXILIARY_CT_STRING_1106061232_HPP

#include <boost/puddle/auxiliary/fwd.hpp>
#include <boost/puddle/auxiliary/base_wrap.hpp>
#include <boost/mirror/ct_string.hpp>

BOOST_PUDDLE_NAMESPACE_BEGIN
namespace aux {

#define BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN(WRAPPER, NAME) \
        static WRAPPER< \
                typename boost::mirror::cts::NAME<CTString>::type \
        > NAME(void) \
        { \
                return WRAPPER< \
                        typename boost::mirror::cts::NAME< \
                                CTString \
                        >::type \
                >(); \
        }

#define BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN(WRAPPER, NAME) \
        template <typename Int, Int Index> \
        static WRAPPER< \
                typename boost::mirror::cts::NAME< \
                        CTString, \
                        std::integral_constant<Int, Index> \
                >::type \
        > NAME(integer<std::integral_constant<Int, Index> >) \
        { \
                return integer< \
                        typename boost::mirror::cts::NAME< \
                                CTString, \
                                std::integral_constant<Int, Index> \
                        >::type \
                >(); \
        }


#define BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN(WRAPPER, NAME) \
        template <int Index> \
        static WRAPPER< \
                typename boost::mirror::cts::NAME<CTString, Index>::type \
        > NAME(void) \
        { \
                return WRAPPER< \
                        typename boost::mirror::cts::NAME< \
                                CTString, \
                                Index \
                        >::type \
                >(); \
        }

#define BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN(WRAPPER, NAME) \
        template <typename CTString_> \
        static WRAPPER< \
                typename boost::mirror::cts::NAME<CTString, CTString_>::type \
        > NAME(ct_string_<CTString_>) \
        { \
                return WRAPPER< \
                        typename boost::mirror::cts::NAME< \
                                CTString, \
                                CTString_ \
                        >::type \
                >(); \
        }

template <typename CTString>
struct ct_string_
{
        static const char* c_str(void)
        {
                return boost::mirror::cts::c_str<CTString>();
        }

        template <typename CTString_>
        friend auto operator == (ct_string_, ct_string_<CTString_>) ->
        integer<
                typename boost::mirror::cts::equal<
                        CTString,
                        CTString_
                >::type
        >
        {
                return integer<
                        typename boost::mirror::cts::equal<
                                CTString,
                                CTString_
                        >::type
                >();
        }

        template <typename CTString_>
        friend auto operator != (ct_string_, ct_string_<CTString_>) ->
        integer<
                typename boost::mirror::cts::nonequal<
                        CTString,
                        CTString_
                >::type
        >
        {
                return integer<
                        typename boost::mirror::cts::nonequal<
                                CTString,
                                CTString_
                        >::type
                >();
        }

        template <typename CTString_>
        friend ct_string_<
                typename boost::mirror::cts::concat<
                        CTString,
                        CTString_
                >::type
        > operator + (ct_string_, ct_string_<CTString_>)
        {
                return ct_string_<
                        typename boost::mirror::cts::concat<
                                CTString,
                                CTString_
                        >::type
                >();
        }

        BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN(integer, empty)
        BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN(integer, size)
        BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN(integer, front)
        BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN(integer, at)
        BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN(integer, at_c)

        BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN(integer, starts_with)
        BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN(integer, ends_with)
        BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN(integer, contains)
        BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN(ct_string_, step_front)
        BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN(ct_string_, skip_front)
        BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN(ct_string_, skip_front_c)
        BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN(ct_string_, head)
        BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN(ct_string_, head_c)
        BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN(ct_string_, tail)
        BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN(ct_string_, tail_c)
        BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN(ct_string_, find)
};

#undef BOOST_PUDDLE_HLPR_IMPL_SIMPLE_CT_STRING_MEM_FN
#undef BOOST_PUDDLE_HLPR_IMPL_IC_PAR_CT_STRING_MEM_FN
#undef BOOST_PUDDLE_HLPR_IMPL_IT_PAR_CT_STRING_MEM_FN
#undef BOOST_PUDDLE_HLPR_IMPL_2S_PAR_CT_STRING_MEM_FN

} // namespace aux

#ifdef BOOST_MIRROR_DOCUMENTATION_ONLY
/// Function for constructing compile-time strings
/**
 *  @see CompileTimeString
 */
template <char ... Chars>
CompileTimeString ct_string(void);
#else
template <char ... Chars>
aux::ct_string_<boost::mirror::cts::string<Chars...> >
ct_string(void)
{
        return aux::ct_string_<boost::mirror::cts::string<Chars...> >();
}
#endif

BOOST_PUDDLE_NAMESPACE_END

#endif //include guard
