/**
 * @file boost/rubber/config.hpp
 * @brief Configuration options
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_RUBBER_CONFIG_1103151301_HPP
#define BOOST_RUBBER_CONFIG_1103151301_HPP

#include <boost/mirror/raw_ptr/raw_ptr.hpp>
#include <boost/mirror/utils/some_type.hpp>

// Macro that begins the Rubber library namespace
#define BOOST_RUBBER_NAMESPACE_BEGIN \
namespace boost { namespace rubber {

// Macro which ends the Rubber library namespace
#define BOOST_RUBBER_NAMESPACE_END \
} /* namespace rubber */ } /* namespace boost */

#ifndef BOOST_RUBBER_POLYMORPHIC_META_OBJECT
#define BOOST_RUBBER_POLYMORPHIC_META_OBJECT 0
#endif

#ifndef BOOST_RUBBER_NO_NAMESPACE_MEMBERS
#define BOOST_RUBBER_NO_NAMESPACE_MEMBERS 0
#endif

#ifndef BOOST_RUBBER_MNSO_WITH_LOCAL_NAME
#define BOOST_RUBBER_MNSO_WITH_LOCAL_NAME 0
#endif

#ifndef BOOST_RUBBER_META_TYPE_OPERATIONS
#define BOOST_RUBBER_META_TYPE_OPERATIONS
#endif

BOOST_RUBBER_NAMESPACE_BEGIN

using boost::mirror::raw_ptr;
using boost::mirror::raw_cast;

using boost::mirror::some;
using boost::mirror::some_ref;
using boost::mirror::some_cast;

namespace aux {

inline int& meta_object_uid_seq(void)
{
        static int seq = 0;
        return seq;
}

template <typename MirrorMetaObject>
inline int meta_object_uid(MirrorMetaObject)
{
        static int uid = ++meta_object_uid_seq();
        return uid;
}

template <typename TypeErased>
struct return_value
{
#if BOOST_RUBBER_POLYMORPHIC_META_OBJECT
        typedef const TypeErased& type;
#else
        typedef TypeErased type;
#endif
};

} // namespace aux

BOOST_RUBBER_NAMESPACE_END

#endif //include guard

