/**
 * @file boost/rubber/meta_meta_object.hpp
 * @brief Wrapper for meta-meta-objects
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_RUBBER_META_META_OBJECT_1103151301_HPP
#define BOOST_RUBBER_META_META_OBJECT_1103151301_HPP

#include <boost/rubber/config.hpp>
#include <boost/mirror/meta_object_tags.hpp>
#include <boost/puddle/meta_object_tags.hpp>

#include <string>

BOOST_RUBBER_NAMESPACE_BEGIN

class meta_meta_object
{
private:
        std::string _base_name;
        std::string _construct_name;

        static inline bool category_matches(unsigned a, unsigned b)
        {
                return (a & b) == b;
        }

        boost::puddle::meta_object_category _cat;
public:
        template <typename MetaMetaObject>
        meta_meta_object(MetaMetaObject mo)
         : _base_name(MetaMetaObject::base_name())
         , _construct_name(MetaMetaObject::construct_name())
         , _cat(boost::puddle::get_category(typename MetaMetaObject::category()))
        { }

        inline std::string base_name(void) const
        {
                return _base_name;
        }

        inline std::string construct_name(void) const
        {
                return _construct_name;
        }

        boost::puddle::meta_object_category category(void) const
        {
                return _cat;
        }

        // Returns true if the reflected meta-object is of the cat category
        inline bool is_a(boost::puddle::meta_object_category cat) const
        {
                return category_matches(
                        unsigned(category()),
                        unsigned(cat)
                );
        }

#define BOOST_RUBBER_HELPER_MAKE_MO_IS_X(OBJECT, I, X) \
        inline bool is_meta_## OBJECT(void) const \
        { \
                return is_a( \
                        boost::puddle::meta_object_category::meta_ ## OBJECT ## _tag \
                ); \
        }
BOOST_MIRROR_FOR_EACH_META_OBJECT(BOOST_RUBBER_HELPER_MAKE_MO_IS_X, _)
#undef BOOST_RUBBER_HELPER_MAKE_MO_IS_X
};

BOOST_RUBBER_NAMESPACE_END

#endif //include guard

