/**
 * @file boost/rubber/specifiers.hpp
 * @brief Wrapper for specifiers
 *
 *  Copyright 2008-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 */

#ifndef BOOST_RUBBER_SPECIFIERS_1103151301_HPP
#define BOOST_RUBBER_SPECIFIERS_1103151301_HPP

#include <boost/rubber/config.hpp>
#include <boost/rubber/range.hpp>
#include <boost/mirror/specifier_tags.hpp>
#include <boost/puddle/specifiers.hpp>

BOOST_RUBBER_NAMESPACE_BEGIN

class specifier
{
private:
        static inline bool category_matches(unsigned a, unsigned b)
        {
                return (a & b) == b;
        }

        boost::puddle::specifier_category _cat;
public:
        template <typename Specifier>
        specifier(Specifier s)
         : _cat(boost::puddle::get_category(s))
        { }

        friend bool operator == (const specifier& a, const specifier& b)
        {
                return a._cat == b._cat;
        }

        friend bool operator != (const specifier& a, const specifier& b)
        {
                return a._cat != b._cat;
        }

        friend bool operator <= (const specifier& a, const specifier& b)
        {
                return a._cat <= b._cat;
        }

        friend bool operator >= (const specifier& a, const specifier& b)
        {
                return a._cat >= b._cat;
        }

        friend bool operator <  (const specifier& a, const specifier& b)
        {
                return a._cat <  b._cat;
        }

        friend bool operator >  (const specifier& a, const specifier& b)
        {
                return a._cat >  b._cat;
        }

        boost::puddle::specifier_category category(void) const
        {
                return _cat;
        }

        inline bool is_a(boost::puddle::specifier_category cat) const
        {
                return category_matches(
                        unsigned(category()),
                        unsigned(cat)
                );
        }

#define BOOST_RUBBER_HELPER_MAKE_SPEC_IS_X(SPEC, KW, GRP, I, X) \
        inline bool is_## SPEC(void) const \
        { \
                return is_a(boost::puddle::specifier_category::spec_ ## SPEC ## _tag); \
        }
BOOST_MIRROR_FOR_EACH_SPECIFIER(BOOST_RUBBER_HELPER_MAKE_SPEC_IS_X, _)
#undef BOOST_RUBBER_HELPER_MAKE_SPEC_IS_X

};

BOOST_RUBBER_NAMESPACE_END

#endif //include guard

